package net.sf.daileon;

import java.io.File;
import java.util.List;

import net.sf.daileon.notsupported.DomainAnnotationsHandler;
import net.sf.daileon.notsupported.DomainAnnotationsXmlReader;
import net.sf.daileon.xml.items.DaileonConfig;
import net.sf.daileon.xml.items.DomainAnnotations;
import net.sf.daileon.xml.items.Setup;

/**
 * The <code>PerformTransformations</code> represents the entry point of the
 * application, and is used when using domain annotations that represent
 * annotations of external frameworks. Given a configuration XML file, starts
 * the translation of the domain annotations of the classes listed in it.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
public class PerformTransformations {

    /**
     * Class constructor.
     */
    public PerformTransformations() {
        super();
    }

    /**
     * Starts the translation of the domain annotations. It first copies the
     * content of the source directory, indicated in the configuration XML file,
     * to the target location, also indicated in the configuration XML file, and
     * then triggers the process of translation of each domain annotation in the
     * classes copied to the target location.
     * 
     * @param args
     *            A <code>String</code> array, that will be passed to this
     *            method through the command line. The value contained in the
     *            first position of this array corresponds to the physical
     *            position of the XML configuration file.
     */
    public static void main(final String [] args) {
        if (args.length == 0) {
            final String message = "Please provide the path to the XML "
                    + "configuration file. For instance:\n\n"
                    + "java -jar daileon.jar C:\\config.xml";
            System.err.print(message);
            System.exit(0);
        }

        String xmlPath = args[0];
        DaileonConfig config = DomainAnnotationsXmlReader.read(xmlPath);

        Setup setup = config.getSetup();
        String rootDirectory = setup.getRootDirectory();
        String destinyDirectory = setup.getDestinyDirectory();

        File root = new File(rootDirectory);
        File destiny = new File(destinyDirectory);

        String finalDestinyPath = destiny.getAbsolutePath()
                + File.separatorChar + root.getName();
        File finalDestiny = new File(finalDestinyPath);

        if (!finalDestiny.exists()) {
            if (!finalDestiny.mkdirs()) {
                String message = "It was not possible to create directory "
                        + finalDestiny.getAbsolutePath()
                        + ". This application cannot continue.";
                throw new RuntimeException(message);
            }
        }

        DirectoryDuplicator.copyDirectory(root, finalDestiny);

        DomainAnnotationsHandler handler = new DomainAnnotationsHandler();

        DomainAnnotations domainAnnotations = config.getDomainAnnotations();
        List<String> domainClasses = domainAnnotations.getAnnotatedClasses();

        handler.makeTransformations(destiny.getAbsolutePath(), domainClasses);
    }
}