package net.sf.daileon;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import net.sf.daileon.notsupported.AbstractAnnotationsReplacer;
import net.sf.daileon.notsupported.DomainAnnotationsXmlReader;
import net.sf.daileon.notsupported.Strategy;
import net.sf.daileon.xml.items.DaileonConfig;

import org.objectweb.asm.ClassVisitor;

/**
 * <p>
 * The <code>ResourcesFactory</code> class is a static factory that retrieves a
 * particular {@link net.sf.daileon.notsupported.Strategy} implementation, or a
 * particular {@link net.sf.daileon.notsupported.AbstractAnnotationsReplacer}
 * implementation.
 * </p>
 * <p>
 * One can define how the annotations that correspond to the domain annotations
 * are organized. If they are organized in annotations, then it is possible to
 * use the default implementation of the <code>Strategy</code> interface, which
 * is the {@link net.sf.daileon.notsupported.AnnotationsStrategy} class. If no
 * values are indicated in the <code>&lt;strategy-class&gt;</code> element of
 * the XML configuration file, then the default implementation is used. However,
 * if they are organized in other forms, such as XML files, or even database
 * tables, then the fully-qualified name of this implementation has to be
 * indicated in the XML configuration file.
 * </p>
 * <p>
 * The user can also define if the domain annotations will be either kept or not
 * in the final bytecode. The framework provides two implementations for each
 * situation, which are the
 * {@link net.sf.daileon.notsupported.DomainAnnotationsKeeper} and the
 * {@link net.sf.daileon.notsupported.DomainAnnotationsDiscarder} classes.
 * However, the user may define situations where the domain annotations should
 * be kept in the final bytecode. To do so, a class that extends the
 * <code>AbstractAnnotationsReplacer</code> has to be built and indicated in the
 * XML configuration file. If no values are indicated in the
 * <code>&lt;domain-annotations-replacer&gt;</code> element of the XML
 * configuration file, then the default implementation is used.
 * </p>
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
public class ResourcesFactory {

    /**
     * The default implementation of the
     * <code>net.sf.daileon.notsupported.Strategy interface. If no values
     * are indicated in the <code>&lt;strategy-class&gt;</code> of the XML
     * configuration file, this value is used by the
     * {@link #getAnnotationsTranslator()} method.
     */
    private static final String DEFAULT_STRATEGY = "net.sf.daileon.notsupported.AnnotationsStrategy";

    /**
     * The default class that extends the
     * {@link net.sf.daileon.notsupported.AbstractAnnotationsReplacer} class. If
     * no values are indicated in the
     * <code>&lt;domain-annotations-replacer&gt;</code> of the XML configuration
     * file, this value is used by the
     * {@link ResourcesFactory #getAnnotationsReplacer(ClassVisitor)} method.
     */
    private static final String DEFAULT_REPLACER = "net.sf.daileon.notsupported.DomainAnnotationsKeeper";

    /**
     * A reference to the {@link net.sf.daileon.notsupported.Strategy} object,
     * indicated in the XML configuration file
     */
    private static Strategy strategy;

    /**
     * A reference to the
     * {@link net.sf.daileon.notsupported.AbstractAnnotationsReplacer} object,
     * indicated in the XML configuration file
     */
    private static AbstractAnnotationsReplacer replacer;

    /**
     * Default class constructor. Has the <code>private</code> constructor,
     * since the <code>ResourcesFactory</code> class is a static factory, and
     * only its static methods should be used.
     */
    private ResourcesFactory() {
        super();
    }

    /**
     * Returns the {@link net.sf.daileon.notsupported.Strategy} object,
     * indicated in the XML configuration file. This object identifies how to
     * translate the domain annotations to their corresponding annotations. If
     * no values are indicated in the XML configuration file, then the default
     * implementation is returned.
     * 
     * @return The <code>Strategy</code> implementation.
     * @throws RuntimeException
     *             If the class provided in the XML configuration file cannot be
     *             instantiated.
     */
    public synchronized static Strategy getAnnotationsTranslator() {
        if (strategy == null) {
            DaileonConfig config = DomainAnnotationsXmlReader
                    .getDaileonConfig();
            String className = config.getSetup().getStrategyClass();

            if (className == null || className.trim().length() == 0) {
                className = DEFAULT_STRATEGY;
            }

            try {
                Object strategyObject = Class.forName(className).newInstance();

                if (!(strategyObject instanceof Strategy)) {
                    String message = "The strategy class informed in the configuration file "
                            + "does not implement interface net.sf.daileon.notsupported.Strategy.";
                    throw new RuntimeException(message);
                }

                strategy = (Strategy) strategyObject;
            } catch (ClassNotFoundException exception) {
                String message = "The following problem occurred while trying "
                        + "to get the implementation of the Strategy interface: "
                        + exception.getMessage();
                throw new RuntimeException(message, exception);
            } catch (IllegalAccessException exception) {
                String message = "The given implementation of the Strategy interface "
                        + "does not have a public no-arg constructor.";
                throw new RuntimeException(message, exception);
            } catch (InstantiationException exception) {
                String message = "It was not possible to instantiate the Strategy "
                        + "interface implementation due to the following problem: ";
                throw new RuntimeException(message, exception);
            }
        }

        return strategy;
    }

    /**
     * Returns the implementation that extends the
     * {@link net.sf.daileon.notsupported.AbstractAnnotationsReplacer}. This
     * object defines whether the domain annotations should be kept or not in
     * the code. If no values are indicated in the XML configuration file, then
     * the default implementation is returned.
     * 
     * @param visitor
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/ClassVisitor.html"
     *            >ClassVisitor</a> object, provided by the
     *            {@link net.sf.daileon.notsupported.ClassTransformer#transform(String, String)}
     *            method.
     * @return The implementation of the
     *         <code>AbstractAnnotationsReplacer</code> class.
     * @throws RuntimeException
     *             If the class provided in the XML configuration file cannot be
     *             instantiated.
     */
    @SuppressWarnings("unchecked")
    public synchronized static AbstractAnnotationsReplacer getAnnotationsReplacer(
            ClassVisitor visitor) {
        DaileonConfig config = DomainAnnotationsXmlReader.getDaileonConfig();
        String className = config.getSetup().getDomainAnnotationsReplacer();

        if (className == null || className.trim().length() == 0) {
            className = DEFAULT_REPLACER;
        }

        try {
            Class clazz = Class.forName(className);
            Constructor constructor = clazz.getConstructor(ClassVisitor.class);

            Object replacerObject = constructor.newInstance(visitor);

            if (!(replacerObject instanceof AbstractAnnotationsReplacer)) {
                String message = "The replacer class informed in the configuration file "
                        + "does not extend the net.sf.daileon.notsupported.AnnotationsReplacer "
                        + " class.";
                throw new RuntimeException(message);
            }

            replacer = (AbstractAnnotationsReplacer) replacerObject;
        } catch (ClassNotFoundException exception) {
            String message = "The following problem occurred while trying "
                    + "to get the replacer class: " + exception.getMessage();
            throw new RuntimeException(message, exception);
        } catch (IllegalAccessException exception) {
            String message = "It was not possible to instantiate the replacer "
                    + "class due to the following problem: ";
            throw new RuntimeException(message, exception);
        } catch (InstantiationException exception) {
            String message = "The given replacer class does not have a public "
                    + "no-arg constructor.";
            throw new RuntimeException(message, exception);
        } catch (NoSuchMethodException exception) {
            String message = "The given replacer class does not have a public "
                    + "constructor that expects a org.objectweb.asm.ClassVisitor "
                    + "object.";
            throw new RuntimeException(message);
        } catch (InvocationTargetException exception) {
            String message = "The following problem occurred while trying "
                    + "to invoke the constructor of the AnnotationsReplacer "
                    + "object: " + exception.getMessage();
            throw new RuntimeException(message);
        }

        return replacer;
    }

}