package net.sf.daileon.notsupported;

import java.util.Iterator;
import java.util.List;

import net.sf.daileon.ResourcesFactory;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.attrs.Annotation;
import org.objectweb.asm.attrs.RuntimeVisibleAnnotations;

/**
 * Essentially, the <code>AbstractAnnotationsCollector</code> is responsible for
 * evaluating the annotations that correspond to a domain annotation and
 * organizing this information in the {@link AbstractAnnotationsReplacer}
 * object, referenced by it. This is mainly performed by the {@link Strategy}
 * object, indicated in the XML configuration file, and used in the
 * {@link AbstractAnnotationsCollector#collect(ElementRoot)} method.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
abstract class AbstractAnnotationsCollector extends ClassAdapter {

    /**
     * The object that performs the actual transformations in the final bytecode
     */
    private AbstractAnnotationsReplacer replacer;

    /** The physical path to the class that corresponds to the domain annotation */
    private String path;

    /**
     * Class constructor.
     * 
     * @param classVisitor
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/ClassVisitor.html"
     *            >ClassVisitor</a> object, provided by the
     *            {@link ClassTransformer#transform(String, String)} method.
     */
    AbstractAnnotationsCollector(final ClassVisitor classVisitor) {
        super(classVisitor);
    }

    /**
     * Sets the <code>AbstractAnnotationsReplacer</code> object. This object is
     * provided by the {@link ClassTransformer#transform(String, String)} method
     * and performs the actual transformations at bytecode level.
     * 
     * @param replacer
     *            The object that performs the actual transformations at
     *            bytecode level.
     */
    void setReplacer(AbstractAnnotationsReplacer replacer) {
        if (replacer == null) {
            String message = "The AnnotationsReplacer argument cannot be null.";
            throw new IllegalArgumentException(message);
        }
        this.replacer = replacer;
    }

    /**
     * Sets the physical path of the class that correspond to the domain
     * annotation.
     * 
     * @param path
     *            The physical path of the domain annotation.
     */
    void setPath(String path) {
        if (path == null) {
            String message = "The path to the class to be transformed cannot "
                    + "be null.";
            throw new IllegalArgumentException(message);
        }
        this.path = path;
    }

    /**
     * Collects the annotations that correspond to a domain annotation, and for
     * such, uses the {@link Strategy} object indicated in the XML configuration
     * file.
     * 
     * @param element
     *            The element that will have its annotations analyzed and
     *            collected. If a domain annotation is found, it will be
     *            evaluated and its corresponding annotations will organized in
     *            the <code>AbstractAnnotationsReplacer</code> object.
     */
    void collect(ElementRoot element) {
        Attribute attribute = element.getAttribute();
        RuntimeVisibleAnnotations annotations = (RuntimeVisibleAnnotations) attribute;
        List<?> annotationsList = annotations.annotations;
        Iterator<?> it = annotationsList.iterator();

        Strategy translator = ResourcesFactory.getAnnotationsTranslator();
        translator.setAnnotationsReplacer(replacer);
        translator.setPath(path);
        translator.setElement(element);

        while (it.hasNext()) {
            Annotation annotation = (Annotation) it.next();
            translator.setAnnotation(annotation);
            translator.execute();
        }
    }
}