package net.sf.daileon.notsupported;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.asm.attrs.Annotation;

/**
 * The <code>AbstractAnnotationsReplacer</code> class is responsible for
 * visiting the elements of a particular class and organizing their annotations.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
public abstract class AbstractAnnotationsReplacer extends
        AbstractAnnotationsReplacerBase {

    /**
     * The name of the class whose elements will have their annotations
     * organized
     */
    private String className;

    /**
     * Class constructor.
     * 
     * @param classVisitor
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/ClassVisitor.html"
     *            >ClassVisitor</a> object, provided by the
     *            {@link ClassTransformer#transform(String, String)} method.
     */
    protected AbstractAnnotationsReplacer(final ClassVisitor classVisitor) {
        super(classVisitor);
    }

    /**
     * Sets the name of the class whose elements will have their annotations
     * organized.
     * 
     * @param className
     *            The name of the class.
     */
    void setClassName(String className) {
        if (className == null) {
            String message = "The name of the class to be transformed "
                    + "cannot be null.";
            throw new IllegalArgumentException(message);
        }
        this.className = className;
    }

    /**
     * Visits the annotations of a given class and calls the
     * {@link #handleElement(ElementRoot)}, so that their corresponding
     * annotations can be organized.
     * 
     * @throws RuntimeException
     *             If a <code>DuplicateAnnotationException</code> is caught when
     *             trying to organize the domain annotations of the class and
     *             their corresponding annotations.
     */
    @Override
    public void visitAttribute(Attribute attribute) {
        ElementRoot element = new ElementRoot();
        element.setClassName(className);
        element.setAttribute(attribute);

        try {
            handleElement(element);
        } catch (DuplicateAnnotationException exception) {
            String message;
            if (exception.getAnnotation() != null) {
                Annotation annotation = exception.getAnnotation();
                String annotationName = RuntimeAnnotationsHelper
                        .getAnnotationName(annotation.type);
                message = "Class " + className
                        + " has two definitions for annotation "
                        + annotationName
                        + ". It is likely that this annotation "
                        + "was defined in a domain annotation and is also "
                        + "annotating class " + className + " directly.";
            } else {
                message = "Class " + className + " has "
                        + "two definitions for the same annotation.";
            }
            throw new RuntimeException(message, exception);
        }

        cv.visitAttribute(attribute);
    }

    /**
     * Visits a particular field of a given class and calls the
     * {@link #handleElement(ElementRoot)}, so that its corresponding
     * annotations can be organized.
     * 
     * @throws RuntimeException
     *             If a <code>DuplicateAnnotationException</code> is caught when
     *             trying to organize the domain annotations of a particular
     *             field and its corresponding annotations.
     */
    @Override
    public void visitField(int access, String name, String desc, Object value,
            Attribute attribute) {

        FieldInformation field = new FieldInformation();
        field.setClassName(className);
        field.setAccess(access);
        field.setName(name);
        field.setDesc(desc);
        field.setValue(value);
        field.setAttribute(attribute);

        try {
            handleElement(field);
        } catch (DuplicateAnnotationException exception) {
            String message;
            if (exception.getAnnotation() != null) {
                Annotation annotation = exception.getAnnotation();
                String annotationName = RuntimeAnnotationsHelper
                        .getAnnotationName(annotation.type);
                message = "Field " + name + " in class " + className
                        + " has two definitions for annotation "
                        + annotationName
                        + ". It is likely that this annotation "
                        + "was defined in a domain annotation and is also "
                        + "annotating field " + name + " directly.";
            } else {
                message = "Field " + name + " has "
                        + "two definitions for the same annotation.";
            }
            throw new RuntimeException(message, exception);
        }

        cv.visitField(access, name, desc, value, attribute);
    }

    /**
     * Visits a particular method of a given class and calls the
     * {@link #handleElement(ElementRoot)}, so that its corresponding
     * annotations can be organized.
     * 
     * @throws RuntimeException
     *             If a <code>DuplicateAnnotationException</code> is caught when
     *             trying to organize the domain annotations of a particular
     *             method and its corresponding annotations.
     */
    @Override
    public CodeVisitor visitMethod(int access, String name, String desc,
            String [] exceptions, Attribute attribute) {

        MethodInformation method = new MethodInformation();
        method.setClassName(className);
        method.setAccess(access);
        method.setName(name);
        method.setDesc(desc);
        method.setExceptions(exceptions);
        method.setAttribute(attribute);

        try {
            handleElement(method);
        } catch (DuplicateAnnotationException exception) {
            String message;
            if (exception.getAnnotation() != null) {
                Annotation annotation = exception.getAnnotation();
                String annotationName = RuntimeAnnotationsHelper
                        .getAnnotationName(annotation.type);
                message = "Method " + name + " in class " + className
                        + " has two definitions for annotation "
                        + annotationName
                        + ". It is likely that this annotation "
                        + "was defined in a domain annotation and is also "
                        + "annotating method " + name + " directly.";
            } else {
                message = "Method " + name + " has "
                        + "two definitions for the same annotation.";
            }
            throw new RuntimeException(message, exception);
        }

        return cv.visitMethod(access, name, desc, exceptions, attribute);
    }

}