package net.sf.daileon.notsupported;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.attrs.Annotation;
import org.objectweb.asm.attrs.RuntimeVisibleAnnotations;

/**
 * The <code>AbstractAnnotationsReplacerBase</code> class is responsible for
 * organizing the elements and their annotations, as well as their domain
 * annotations.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
abstract class AbstractAnnotationsReplacerBase extends ClassAdapter {

    /** Keeps the elements and their annotations */
    private Map<ElementRoot, RuntimeVisibleAnnotations> elementsMap = new HashMap<ElementRoot, RuntimeVisibleAnnotations>();

    /** Keeps the elements and their domain annotations */
    private Map<ElementRoot, Set<String>> domainAnnotations = new HashMap<ElementRoot, Set<String>>();

    /**
     * Informs whether the domain annotations should be either kept or not in
     * the final bytecode.
     * 
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the domain annotations should be kept in
     *         the final bytecode;
     *         <li><code>false</code> if the domain annotations should not be
     *         kept in the final bytecode.
     *         </ul>
     */
    protected abstract boolean keepDomainAnnotations();

    /**
     * Class constructor.
     * 
     * @param visitor
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/ClassVisitor.html"
     *            >ClassVisitor</a> object, provided by the
     *            {@link ClassTransformer#transform(String, String)} method.
     */
    AbstractAnnotationsReplacerBase(ClassVisitor visitor) {
        super(visitor);
    }

    /**
     * Associates the name of a domain annotation to a particular element.
     * 
     * @param element
     *            The element which the domain annotation is associated to.
     * @param annotation
     *            The name of the domain annotation.
     */
    void addDomainAnnotation(ElementRoot element, String annotation) {
        Set<String> domainAnnotationsSet = domainAnnotations.get(element);

        if (domainAnnotationsSet == null) {
            domainAnnotationsSet = new HashSet<String>();
            domainAnnotations.put(element, domainAnnotationsSet);
        }

        domainAnnotationsSet.add(annotation);
    }

    /**
     * Associates annotations to a particular element.
     * 
     * @param element
     *            The element which the annotations are associated to.
     * @param annotations
     *            The <a href="http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/attrs/RuntimeInvisibleAnnotations.html"
     *            >RuntimeVisibleAnnotations</a> object, containing the
     *            annotations to be associated with the element.
     */
    void addElementAnnotations(ElementRoot element,
            RuntimeVisibleAnnotations annotations) {

        elementsMap.put(element, annotations);
    }

    /**
     * Organizes the elements and their annotations. Essentially, it first
     * verifies if the domain annotations should be kept in the code; if they
     * should not be kept, then the domain annotations that annotate the element
     * in the bytecode are removed from the <code>Map</code> that associates
     * elements with domain annotations. After that, it verifies if two or more
     * domain annotations evaluate to the same annotation. If so, a
     * <code>DuplicateAnnotationException</code> is thrown.
     * 
     * @param element
     *            The element to have its annotations organized.
     * @throws DuplicateAnnotationException
     *             If two or more domain annotations evaluate to the same
     *             annotation.
     */
    @SuppressWarnings("unchecked")
    void handleElement(ElementRoot element) throws DuplicateAnnotationException {

        Attribute attribute = element.getAttribute();
        if (attribute != null && attribute instanceof RuntimeVisibleAnnotations) {
            RuntimeVisibleAnnotations finalAnnotations = (RuntimeVisibleAnnotations) element
                    .getAttribute();
            List<?> finalAnnotationsList = finalAnnotations.annotations;

            if (!keepDomainAnnotations()) {
                Iterator<?> iterator = finalAnnotationsList.iterator();

                while (iterator.hasNext()) {
                    Annotation annotation = (Annotation) iterator.next();
                    String nextAnnotation = RuntimeAnnotationsHelper
                            .getAnnotationName(annotation.type);

                    Set<String> annotationsSet = domainAnnotations.get(element);

                    if (annotationsSet != null) {
                        Iterator<String> domainAnnotationsIterator = annotationsSet
                                .iterator();

                        while (domainAnnotationsIterator.hasNext()) {
                            String domainAnnotation = (String) domainAnnotationsIterator
                                    .next();
                            if (domainAnnotation.equals(nextAnnotation)) {
                                iterator.remove();
                                domainAnnotationsIterator.remove();
                            }
                        }
                    }
                }
            }

            RuntimeVisibleAnnotations annotations = (RuntimeVisibleAnnotations) elementsMap
                    .get(element);
            List<?> finalList = annotations.annotations;
            Iterator<?> finalListIterator = finalList.iterator();

            while (finalListIterator.hasNext()) {
                Annotation extraAnnotation = (Annotation) finalListIterator
                        .next();
                Iterator<?> itFinalAnnotations = finalAnnotationsList
                        .iterator();

                while (itFinalAnnotations.hasNext()) {
                    Annotation finalAnnotation = (Annotation) itFinalAnnotations
                            .next();
                    if (finalAnnotation.type.equals(extraAnnotation.type)) {
                        throw new DuplicateAnnotationException(finalAnnotation);
                    }
                }
            }

            finalAnnotationsList.addAll(annotations.annotations);
        }
    }
}