package net.sf.daileon.notsupported;

import java.util.Iterator;
import java.util.List;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.attrs.Annotation;
import org.objectweb.asm.attrs.RuntimeVisibleAnnotations;

/**
 * The <code>AbstractTemplateAnnotationsCollector</code> class is responsible
 * for collecting the annotations of the elements indicated in template
 * annotations, such as {@link net.sf.daileon.annotation.FieldTemplate} or
 * {@link net.sf.daileon.annotation.MethodTemplate}.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
abstract class AbstractTemplateAnnotationsCollector extends ClassAdapter {

    /**
     * The fully-qualified name of the {@link java.lang.annotation.Documented}
     * annotation
     */
    private static final String DOCUMENTED = "java.lang.annotation.Documented";

    /**
     * The fully-qualified name of the {@link java.lang.annotation.Retention}
     * annotation
     */
    private static final String RETENTION = "java.lang.annotation.Retention";

    /**
     * The fully-qualified name of the {@link java.lang.annotation.Target}
     * annotation
     */
    private static final String TARGET = "java.lang.annotation.Target";

    /**
     * The fully-qualified name of the
     * {@link net.sf.daileon.annotation.DomainAnnotation} annotation
     */
    private static final String DOMAIN_ANNOTATION = "net.sf.daileon.annotation.DomainAnnotation";

    /**
     * The element that keeps the annotations that are collected from a
     * particular element, which is indicated in a template annotation
     */
    private RuntimeVisibleAnnotations annotations = new RuntimeVisibleAnnotations();

    /**
     * Class constructor.
     * 
     * @param classVisitor
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/ClassVisitor.html"
     *            >ClassVisitor</a> object that visits the class.
     */
    AbstractTemplateAnnotationsCollector(ClassVisitor classVisitor) {
        super(classVisitor);
    }

    /**
     * Gets the object that keeps the annotations indicated in a class or class
     * element of a template annotation.
     * 
     * @return The object that keeps the annotations indicated in a class or
     *         class element of a template annotation.
     */
    RuntimeVisibleAnnotations getAnnotations() {
        return annotations;
    }

    /**
     * Sets the object that keeps the annotations indicated in a class or class
     * element of a template annotation.
     * 
     * @param annotations
     *            The object that keeps the annotations indicated in a class or
     *            class element of a template annotation.
     */
    void setAnnotations(RuntimeVisibleAnnotations annotations) {
        this.annotations = annotations;
    }

    /**
     * Collects the annotations of a given <a href=
     * "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/Attribute.html"
     * >Attribute</a>, which can contain one or several annotations.
     * 
     * @param attribute
     *            The <code>Attribute</code> object, containing one or several
     *            annotations to be collected. This object always corresponds to
     *            the annotations of an element indicated in a template
     *            annotation.
     */
    @SuppressWarnings("unchecked")
    void collectAnnotations(Attribute attribute) {
        RuntimeVisibleAnnotations annotations = (RuntimeVisibleAnnotations) attribute;
        List<?> annotationsList = annotations.annotations;
        Iterator<?> it = annotationsList.iterator();
        while (it.hasNext()) {
            Annotation annotation = (Annotation) it.next();
            if (isValidAnnotation(annotation)) {
                this.annotations.annotations.add(annotation);
            }
        }
    }

    /**
     * Informs whether an annotation is valid or not. If it is the
     * {@link java.lang.annotation.Documented}, or
     * {@link java.lang.annotation.Retention}, or
     * {@link java.lang.annotation.Target} or
     * {@link net.sf.daileon.annotation.DomainAnnotation}, then it is not valid.
     * 
     * @param annotation
     *            The <code>Annotation</code> object to be verified.
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the annotation is valid;</li>
     *         <li><code>false</code> if the annotation is not valid.</li>
     *         </ul>
     */
    private boolean isValidAnnotation(Annotation annotation) {
        String annotationName = RuntimeAnnotationsHelper
                .getAnnotationName(annotation.type);
        boolean isValidAnnotation = true;
        if (annotationName.equals(DOCUMENTED)
                || annotationName.equals(RETENTION)
                || annotationName.equals(TARGET)
                || annotationName.equals(DOMAIN_ANNOTATION)) {
            isValidAnnotation = false;
        }
        return isValidAnnotation;
    }

}