package net.sf.daileon.notsupported;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.asm.attrs.RuntimeVisibleAnnotations;

/**
 * The <code>AnnotationsCollector</code> class is responsible for visiting the
 * annotations of a particular class, as well as its fields and methods.<br />
 * Classes visited by the <code>AnnotationsCollector</code> should have domain
 * annotations, which are evaluated to their corresponding annotations and
 * prepared for transformations at bytecode level.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
class AnnotationsCollector extends AbstractAnnotationsCollector {

    /** The name of the class to be visted */
    private String className;

    /**
     * Class constructor.
     * 
     * @param classVisitor
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/ClassVisitor.html"
     *            >ClassVisitor</a> object, provided by the
     *            {@link ClassTransformer#transform(String, String)} method.
     */
    AnnotationsCollector(final ClassVisitor classVisitor) {
        super(classVisitor);
    }

    /**
     * Sets the name of the class to be visited.
     * 
     * @param className
     *            The name of the class to be visited.
     */
    void setClassName(String className) {
        if (className == null) {
            String message = "The name of the class to be transformed "
                    + "cannot be null.";
            throw new IllegalArgumentException(message);
        }
        this.className = className;
    }

    /**
     * Visits the annotations of a given class, which will be verified. If
     * domain annotations are found, they are evaluated to their corresponding
     * annotations and prepared for transformations at bytecode level.
     * 
     * @param attribute
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/Attribute.html"
     *            >Attribute</a> object (or the annotations at class level),
     *            which may contain one or several annotations.
     */
    @Override
    public void visitAttribute(Attribute attribute) {
        if (attribute instanceof RuntimeVisibleAnnotations) {
            ElementRoot element = new ElementRoot();
            element.setClassName(className);
            element.setAttribute(attribute);

            collect(element);
        }
    }

    /**
     * Visits a field of a given class. If domain annotations are found on it,
     * they are evaluated to their corresponding annotations and prepared for
     * transformations at bytecode level.
     * 
     * @param access
     *            The access level of the field.
     * @param name
     *            The name of the field.
     * @param desc
     *            The description of the field.
     * @param value
     *            The default value of the field.
     * @param attribute
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/Attribute.html"
     *            >Attribute</a> object (or the annotations that annotate the
     *            field), which may contain one or several annotations.
     */
    @Override
    public void visitField(int access, String name, String desc, Object value,
            Attribute attribute) {
        if (attribute instanceof RuntimeVisibleAnnotations) {
            FieldInformation field = new FieldInformation();
            field.setClassName(className);
            field.setAccess(access);
            field.setName(name);
            field.setDesc(desc);
            field.setValue(value);
            field.setAttribute(attribute);

            collect(field);
        }
    }

    /**
     * Visits a method of a given class. If domain annotations are found on it,
     * they are evaluated to their corresponding annotations and prepared for
     * transformations at bytecode level.
     * 
     * @param access
     *            The access level of the field.
     * @param name
     *            The name of the field.
     * @param desc
     *            The description of the field.
     * @param exceptions
     *            The exceptions declared by the method.
     * @param attribute
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/Attribute.html"
     *            >Attribute</a> object (or the annotations that annotate the
     *            method), which may contain one or several annotations.
     * @return <code>null</code>, since the method is only visited so its
     *         annotations can be evaluated and organized properly, if domain
     *         annotations are found on it.
     */
    @Override
    public CodeVisitor visitMethod(int access, String name, String desc,
            String [] exceptions, Attribute attribute) {

        if (attribute instanceof RuntimeVisibleAnnotations) {
            MethodInformation method = new MethodInformation();
            method.setClassName(className);
            method.setAccess(access);
            method.setName(name);
            method.setDesc(desc);
            method.setExceptions(exceptions);
            method.setAttribute(attribute);

            collect(method);
        }
        return null;
    }

}