package net.sf.daileon.notsupported;

import org.objectweb.asm.attrs.Annotation;
import org.objectweb.asm.attrs.RuntimeVisibleAnnotations;

/**
 * The <code>AnnotationsStrategy</code> class assumes that the annotations that
 * correspond to domain annotations are organized in class elements, indicated
 * in template annotations that annotate the domain annotations.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
public class AnnotationsStrategy implements Strategy {

    /**
     * The <code>AbstractAnnotationsReplacer</code> object, that organizes the
     * elements and their annotations, as well as their domain annotations
     */
    private AbstractAnnotationsReplacer annotationsReplacer;

    /** The domain annotation to be evaluated */
    private Annotation annotation;

    /** The physical path to the domain annotation */
    private String path;

    /** The element annotated with the domain annotation */
    private ElementRoot element;

    /**
     * Default class constructor.
     */
    public AnnotationsStrategy() {
        super();
    }

    /** {@inheritDoc} */
    public void setAnnotationsReplacer(AbstractAnnotationsReplacer replacer) {
        this.annotationsReplacer = replacer;
    }

    /** {@inheritDoc} */
    public void setAnnotation(Annotation annotation) {
        this.annotation = annotation;
    }

    /** {@inheritDoc} */
    public void setPath(String path) {
        this.path = path;
    }

    /** {@inheritDoc} */
    public void setElement(ElementRoot element) {
        this.element = element;
    }

    /** {@inheritDoc} */
    public void execute() {
        if (annotationsReplacer == null) {
            String message = "Before transforming a domain annotation, it is "
                    + "necessary to define and set a AnnotationsReplacer in the "
                    + "class that executes the transformations.";
            throw new IllegalStateException(message);
        } else if (annotation == null) {
            String message = "Before transforming a domain annotation, it is "
                    + "necessary to define and set an Annotation in the class that "
                    + "executes the transformations.";
            throw new IllegalStateException(message);
        } else if (path == null) {
            String message = "The annotation path must be set before executing "
                    + "the transformations.";
            throw new IllegalStateException(message);
        } else if (element == null) {
            String message = "The element that contains the domain annotation "
                    + "must be set before executing the transformations.";
            throw new IllegalStateException(message);
        }

        if (RuntimeAnnotationsHelper.isDomainAnnotation(path, annotation)) {
            String annotationName = RuntimeAnnotationsHelper
                    .getAnnotationName(annotation.type);
            annotationsReplacer.addDomainAnnotation(element, annotationName);
            try {
                RuntimeVisibleAnnotations annotations = RuntimeAnnotationsHelper
                        .retrieveAnnotations(path, annotation);
                annotationsReplacer.addElementAnnotations(element, annotations);
            } catch (StackOverflowError error) {
                String message = "Warning: domain annotation "
                        + annotation
                        + " could not be translated to its corresponding annotations "
                        + "due to the following error: " + error.getMessage();
                throw new RuntimeException(message, error);
            } catch (DuplicateAnnotationException exception) {
                String message;
                if (exception.getAnnotation() != null) {
                    message = "Domain annotation " + annotation.type + " has "
                            + "two definitions for annotation "
                            + exception.getAnnotation().type + ".";
                } else {
                    message = "Domain annotation " + annotation.type
                            + " has two definitions for the same annotation.";
                }
                throw new RuntimeException(message, exception);
            }
        }
    }
}