package net.sf.daileon.notsupported;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import net.sf.daileon.ResourcesFactory;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.attrs.Attributes;

/**
 * The <code>ClassTransformer</code> is responsible for triggering the process
 * of translating the domain annotations of a particular class.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
class ClassTransformer {

    /**
     * Class constructor.
     */
    ClassTransformer() {
        super();
    }

    /**
     * Triggers the process translating the domain annotations of a given
     * class.
     * 
     * @param path
     *            The physical path to the class containing the domain
     *            annotations, excluding its package. For instance, if a class
     *            called <code>com.project.MyClass</code> is located at
     *            <code>C:\MyProject</code>, then the physical path is
     *            <code>C:\MyProject</code>.
     * @param annotatedClass
     *            The fully-qualified name of the class whose domain annotations
     *            shall be evaluated.
     * @throws RuntimeException
     *             If any problem occur when trying to read the class that
     *             contains the domain annotations, or when trying to write the
     *             classes with the domain annotations translated.
     */
    void transform(String path, String annotatedClass) {
        if (path == null) {
            String message = "It is necessary to indicate the path where the "
                    + "class to be transformed are located.";
            throw new IllegalArgumentException(message);
        } else if (annotatedClass == null) {
            String message = "It is necessary to provide the class that contains "
                    + "the domain annotations to be replaced.";
            throw new IllegalArgumentException(message);
        }

        String className = RuntimeAnnotationsHelper.getClassPath(path,
                annotatedClass);

        try {
            InputStream stream = new FileInputStream(className);
            ClassReader reader = new ClassReader(stream);
            ClassWriter writer = new ClassWriter(false);

            AbstractAnnotationsReplacer replacer = ResourcesFactory
                    .getAnnotationsReplacer(writer);
            replacer.setClassName(className);

            AnnotationsCollector collector = new AnnotationsCollector(
                    new ClassWriter(false));
            collector.setClassName(className);
            collector.setPath(path);
            collector.setReplacer(replacer);

            reader.accept(collector, Attributes.getDefaultAttributes(), true);
            reader.accept(replacer, Attributes.getDefaultAttributes(), true);

            byte [] modifiedClass = writer.toByteArray();
            FileOutputStream output = new FileOutputStream(className);
            output.write(modifiedClass);
            output.flush();
            output.close();

            String message = "Domain annotations of class " + annotatedClass
                    + " were replaced successfully.";
            System.out.println(message);
        } catch (FileNotFoundException exception) {
            String message = "Class " + annotatedClass + " could not be found "
                    + "in " + path;
            throw new RuntimeException(message, exception);
        } catch (IOException exception) {
            String message = "It was not possible to replace domain annotations "
                    + "of class "
                    + annotatedClass
                    + " due to the following "
                    + "error: " + exception.getMessage();
            throw new RuntimeException(message, exception);
        }
    }
}