package net.sf.daileon.notsupported;

import java.util.Iterator;
import java.util.List;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.attrs.Annotation;
import org.objectweb.asm.attrs.RuntimeVisibleAnnotations;

/**
 * The <code>DomainAnnotationVerifier</code> class is responsible for verifying
 * if an annotation is a domain annotation, that is, if it is annotated with the
 * {@link net.sf.daileon.annotation.DomainAnnotation} annotation.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
class DomainAnnotationVerifier extends ClassAdapter {

    /**
     * The fully-qualified name of the <code>@DomainAnnotatino</code> annotation
     */
    private static final String DOMAIN_ANNOTATION_NAME = "net.sf.daileon.annotation.DomainAnnotation";

    /** Informs whether the annotation is a domain annotation */
    private boolean isDomainAnnotation = false;

    /**
     * Class constructor.
     * 
     * @param classVisitor
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/ClassVisitor.html"
     *            >ClassVisitor</a> object that visits the class.
     */
    DomainAnnotationVerifier(ClassVisitor classVisitor) {
        super(classVisitor);
    }

    /**
     * Analyzes the annotations of a given annotation. If a
     * <code>@DomainAnnotation</code> is found, then the
     * <code>isDomainAnnotation</code> variable is set to <code>true</code>.
     * 
     * @param attribute
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/Attribute.html"
     *            >Attribute</a> object, which may contain one or several
     *            annotations.
     */
    @Override
    public void visitAttribute(Attribute attribute) {
        if (attribute instanceof RuntimeVisibleAnnotations) {
            RuntimeVisibleAnnotations annotations = (RuntimeVisibleAnnotations) attribute;
            List<?> annotationsList = annotations.annotations;
            Iterator<?> it = annotationsList.iterator();
            while (it.hasNext()) {
                Annotation annotation = (Annotation) it.next();
                String annotationName = RuntimeAnnotationsHelper
                        .getAnnotationName(annotation.type);
                if (annotationName.equals(DOMAIN_ANNOTATION_NAME)) {
                    isDomainAnnotation = true;
                    break;
                }
            }
        }
    }

    /**
     * Informs whether the annotation is a domain annotation.
     * 
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the annotation is a domain annotation;</li>
     *         <li><code>false</code> if the annotation is not a domain
     *         annotation.</li>
     *         </ul>
     */
    boolean isDomainAnnotation() {
        return isDomainAnnotation;
    }

}