package net.sf.daileon.notsupported;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import net.sf.daileon.xml.XmlHandler;
import net.sf.daileon.xml.XmlHandlerException;
import net.sf.daileon.xml.XmlHandlerFactory;
import net.sf.daileon.xml.items.DaileonConfig;

/**
 * The <code>DomainAnnotationsXmlReader</code> class is responsible for dealing
 * with XML configuration file, indicated in the command line when Daileon is
 * run.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
public class DomainAnnotationsXmlReader {

    /**
     * The package where the elements that contain the objects annotated with
     * annotations of the <code>JAXB</code> API can be found
     */
    private static final String JAXB_CONTEXT = "net.sf.daileon.xml.items";

    /**
     * The <code>DaileonConfig</code> object, that is, the root element of the
     * XML configuration file
     */
    private static DaileonConfig daileonConfig;

    /**
     * Class constructor.
     */
    private DomainAnnotationsXmlReader() {
        super();
    }

    /**
     * Reads a given XML file, located in the indicated location.
     * 
     * @param xmlLocation
     *            The complete physical path of the XML configuration file.
     * @return The <code>DaileonConfig</code> object that corresponds to the
     *         root element of the indicated XML configuration file.
     */
    public static DaileonConfig read(String xmlLocation) {

        InputStream inputStream;

        try {
            inputStream = new FileInputStream(xmlLocation);
        } catch (FileNotFoundException exception) {
            final String message = "The given file does not exist.";
            throw new RuntimeException(message, exception);
        }

        final XmlHandler xmlHandler = XmlHandlerFactory.getJaxbXmlHandler();

        try {
            daileonConfig = (DaileonConfig) xmlHandler.xmlToBean(inputStream,
                    JAXB_CONTEXT);
        } catch (final XmlHandlerException exception) {
            final String message = "The following error occurred while trying "
                    + "to read the XML configuration file: "
                    + exception.getMessage();
            throw new RuntimeException(message, exception);
        }

        return daileonConfig;
    }

    /**
     * Retrieves the <code>DaileonConfig</code> object that corresponds to the
     * root element of the indicated XML configuration file.
     * 
     * @return The <code>DaileonConfig</code> object that corresponds to the
     *         root element of the indicated XML configuration file. The
     *         <code>read</code> method must be called prior to this method.
     * @throws IllegalStateException
     *             If the <code>read</code> method was not called prior to this
     *             method.
     */
    public static DaileonConfig getDaileonConfig() {
        if (daileonConfig == null) {
            String message = "The read(String) method must be called "
                    + "prior to the getDaileonConfig() method.";
            throw new IllegalStateException(message);
        }
        return daileonConfig;
    }

}