package net.sf.daileon.notsupported;

import org.objectweb.asm.attrs.Annotation;

/**
 * The <code>DuplicateAnnotationException</code> is thrown when a domain
 * annotation is evaluated to 2 equal annotations. This is likely to happen when
 * a domain annotation is annotated with other domain annotation, or when the
 * element has the annotation, and the domain annotation evaluates to the same
 * annotation.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
public class DuplicateAnnotationException extends Exception {

    /** serialVersionUID */
    private static final long serialVersionUID = 1491853661111656177L;

    /** The <code>Annotation</code> duplicated object */
    private Annotation annotation;

    /**
     * Constructs a new exception with <code>null</code> as its detail message.
     * The cause is not initialized, and may subsequently be initialized by a
     * call to {@link #initCause}.
     */
    public DuplicateAnnotationException() {
        super();
    }

    /**
     * Constructs a new exception with <code>null</code> as its detail message
     * and a <code>Annotation</code> object, representing the duplicate
     * annotation. The cause is not initialized, and may subsequently be
     * initialized by a call to {@link #initCause}.
     */
    public DuplicateAnnotationException(Annotation annotation) {
        super();
        this.annotation = annotation;
    }

    /**
     * Constructs a new exception with the specified detail message, cause and
     * the duplicate annotation.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this exception's detail message.
     * 
     * @param message
     *            The detail message (which is saved for later retrieval by the
     *            {@link #getMessage()} method).
     * @param cause
     *            The cause (which is saved for later retrieval by the
     *            {@link #getCause()} method). (A <tt>null</tt> value is
     *            permitted, and indicates that the cause is nonexistent or
     *            unknown.)
     * @param annotation
     *            The duplicate annotation.
     */
    public DuplicateAnnotationException(Annotation annotation, String message,
            Throwable cause) {
        super(message, cause);
        this.annotation = annotation;
    }

    /**
     * Constructs a new exception with the specified detail message and the
     * duplicate annotation. The cause is not initialized, and may subsequently
     * be initialized by a call to {@link #initCause}.
     * 
     * @param annotation
     *            The duplicate annotation.
     * @param message
     *            The detail message. The detail message is saved for later
     *            retrieval by the {@link #getMessage()} method.
     */
    public DuplicateAnnotationException(Annotation annotation, String message) {
        super(message);
        this.annotation = annotation;
    }

    /**
     * Constructs a new exception with the duplicate annotation, the specified
     * cause and a detail message of
     * <code>(cause == null ? null : cause.toString())</code> (which typically
     * contains the class and detail message of <code>cause</code>).
     * 
     * @param annotation
     *            The duplicate annotation.
     * @param cause
     *            the cause (which is saved for later retrieval by the
     *            {@link #getCause()} method). (A <tt>null</tt> value is
     *            permitted, and indicates that the cause is nonexistent or
     *            unknown.)
     */
    public DuplicateAnnotationException(Annotation annotation, Throwable cause) {
        super(cause);
        this.annotation = annotation;
    }

    /**
     * Gets the duplicate annotation.
     * 
     * @return The duplicate annotation.
     */
    public Annotation getAnnotation() {
        return annotation;
    }

}