package net.sf.daileon.notsupported;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.asm.attrs.RuntimeVisibleAnnotations;

/**
 * The <code>EquivalentAnnotationsCollector</code> is responsible for collecting
 * the annotations of the elements indicated in the template classes, which are
 * the elements that keep annotations that correspond to domain annotations. It
 * reads annotations of classes, fields or methods, according to the element
 * indicated in a particular template annotation.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 * @see net.sf.daileon.annotation.ClassTemplate
 * @see net.sf.daileon.annotation.FieldTemplate
 * @see net.sf.daileon.annotation.MethodTemplate
 */
class EquivalentAnnotationsCollector extends
        AbstractTemplateAnnotationsCollector {

    /**
     * The field name to be read, as indicated in a <code>FieldTemplate</code>
     * annotation
     */
    private String fieldName;

    /**
     * The method name to be read, as indicated in a <code>MethodTemplate</code>
     * annotation
     */
    private String methodName;

    /**
     * Informs whether the annotations that correspond to a domain annotation
     * are at class level
     */
    private boolean classAnnotations;

    /**
     * Controls the visiting of the element indicated in the template annotation
     */
    private boolean elementVisited = false;

    /**
     * Class constructor.
     * 
     * @param classVisitor
     *            The <a href=
     *            "http://asm.ow2.org/asm153/javadoc/user/org/objectweb/asm/ClassVisitor.html"
     *            >ClassVisitor</a> object that visits the class.
     */
    EquivalentAnnotationsCollector(ClassVisitor classVisitor) {
        super(classVisitor);
    }

    /**
     * Gets the field name to be visited. This field keeps the annotations that
     * are indicated in a particular <code>FieldTemplate</code> annotation.
     * 
     * @return The field name.
     */
    String getFieldName() {
        return fieldName;
    }

    /**
     * Sets the field name to be visited. This field keeps the annotations that
     * are indicated in a particular <code>FieldTemplate</code> annotation.
     * 
     * @param fieldName
     *            The field name.
     */
    void setFieldName(String fieldName) {
        this.fieldName = fieldName;
    }

    /**
     * Gets the method name to be visited. This method keeps the annotations
     * that are indicated in a particular <code>MethodTemplate</code>
     * annotation.
     * 
     * @return The method name.
     */
    String getMethodName() {
        return methodName;
    }

    /**
     * Sets the method name to be visited. This method keeps the annotations
     * that are indicated in a particular <code>MethodTemplate</code>
     * annotation.
     * 
     * @param methodName
     *            The method name.
     */
    void setMethodName(String methodName) {
        this.methodName = methodName;
    }

    /**
     * Informs whether the annotations are at class level.
     * 
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the annotations indicated by a template
     *         annotation are at class level;</li>
     *         <li><code>false</code> if the annotations indicated are not at
     *         class level.</li>
     *         </ul>
     */
    boolean isClassAnnotations() {
        return classAnnotations;
    }

    /**
     * Defines whether the annotations that correspond to a particular domain
     * annotation are at class level.
     * 
     * @param classAnnotations
     *            A <code>boolean</code> value that can be:
     *            <ul>
     *            <li><code>true</code> if the annotations to be evaluated are
     *            at class level;</li>
     *            <li><code>false</code> if the annotations to be evaluated are
     *            not at class level.</li>
     *            </ul>
     */
    void setClassAnnotations(boolean classAnnotations) {
        this.classAnnotations = classAnnotations;
    }

    /**
     * Informs whether the element that keeps the annotations that correspond to
     * a particular domain annotation has already been visited.
     * 
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the element has already been analyzed;</li>
     *         <li><code>false</code> if the element has not been analyzed.</li>
     *         </ul>
     */
    boolean isElementVisited() {
        return elementVisited;
    }

    /**
     * Collects the annotations of a particular class at class level.
     */
    @Override
    public void visitAttribute(Attribute attribute) {
        if (attribute instanceof RuntimeVisibleAnnotations && classAnnotations) {
            collectAnnotations(attribute);
        }
    }

    /**
     * Collects the annotations of a particular particular field, which is
     * indicated via the {@link #setFieldName(String)} method.
     */
    @Override
    public void visitField(int access, String name, String desc, Object value,
            Attribute attribute) {
        if (name.equals(fieldName)) {
            elementVisited = true;
            collectAnnotations(attribute);
        }
    }

    /**
     * Collects the annotations of a particular method, which is indicated via
     * the {@link #setMethodName(String)} method.
     */
    @Override
    public CodeVisitor visitMethod(int access, String name, String desc,
            String [] exceptions, Attribute attribute) {
        if (name.equals(methodName)) {
            elementVisited = true;
            collectAnnotations(attribute);
        }
        return null;
    }

}