package net.sf.daileon.notsupported;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.attrs.Annotation;
import org.objectweb.asm.attrs.Attributes;
import org.objectweb.asm.attrs.RuntimeVisibleAnnotations;

/**
 * The <code>RuntimeAnnotationsHelper</code> class provides several static
 * methods for dealing with annotations. This class is useful for the default
 * approach of keeping the annotations that correspond to each domain annotation
 * in elements indicated in template annotations.
 * 
 * @author Roberto Perillo
 * @version 1.0 01/01/2010
 */
class RuntimeAnnotationsHelper {

    /** The fully-qualified name of the <code>ClassTemplate</code> annotation */
    private static final String CLASS_TEMPLATE = "net.sf.daileon.annotation.ClassTemplate";

    /** The fully-qualified name of the <code>MethodTemplate</code> annotation */
    private static final String METHOD_TEMPLATE = "net.sf.daileon.annotation.MethodTemplate";

    /** The fully-qualified name of the <code>FieldTemplate</code> annotation */
    private static final String FIELD_TEMPLATE = "net.sf.daileon.annotation.FieldTemplate";

    /**
     * Class constructor.
     */
    private RuntimeAnnotationsHelper() {
        super();
    }

    /**
     * Informs whether a given annotation is a domain annotation.
     * 
     * @param path
     *            The path to the annotation.
     * @param annotation
     *            The annotation to be verified.
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the given annotation is a domain
     *         annotation;</li>
     *         <li><code>false</code> if the given annotation is not a domain
     *         annotation.</li>
     *         </ul>
     */
    static boolean isDomainAnnotation(String path, Annotation annotation) {
        String resource = getResourcePath(path, annotation);

        DomainAnnotationVerifier verifier = new DomainAnnotationVerifier(
                new ClassWriter(true));

        try {
            InputStream stream = new FileInputStream(resource);
            ClassReader reader = new ClassReader(stream);
            reader.accept(verifier, Attributes.getDefaultAttributes(), true);
            return verifier.isDomainAnnotation();
        } catch (FileNotFoundException exception) {
            return false;
        } catch (IOException e) {
            String message = "The following error occurred while trying to verify "
                    + "if annotation "
                    + annotation.type
                    + " is a domain "
                    + "annotation: " + e.getMessage();
            System.err.println(message);
            return false;
        }
    }

    /**
     * Gets the name (in the format package.name) of a particular annotation.
     * 
     * @param annotation
     *            The name of the annotation.
     * @return The name of the annotation, in the format package.name.
     */
    static String getAnnotationName(String annotation) {
        String annotationName = annotation.replace('/', '.');

        if (annotationName.startsWith("@")) {
            annotationName = annotationName.substring(1);
        }

        if (annotationName.startsWith("L")) {
            annotationName = annotationName.substring(1);
        }

        if (annotationName.endsWith(";")) {
            annotationName = annotationName.substring(0, annotationName
                    .length() - 1);
        }

        return annotationName;
    }

    /**
     * Retrieves the annotations that correspond to a domain annotation.
     * Essentially, it reads the domain annotation, reads the elements indicated
     * in it and collects the annotations that annotate them.
     * 
     * @param path
     *            The path to the domain annotation to be evaluated.
     * @param annotation
     *            The domain annotation to be evaluated.
     * @return The <a href="">RuntimeVisibleAnnotations</a>, containing the
     *         annotations that correspond to the domain annotation.
     * @throws DuplicateAnnotationException
     *             If the domain annotation evaluates to an annotation that
     *             already annotates the element, or if a domain annotation
     *             annotates another domain annotation, and both of them
     *             evaluate to the same annotation.
     */
    @SuppressWarnings("unchecked")
    static RuntimeVisibleAnnotations retrieveAnnotations(String path,
            Annotation annotation) throws DuplicateAnnotationException {

        String resource = getResourcePath(path, annotation);

        RuntimeVisibleAnnotations finalAnnotations = new RuntimeVisibleAnnotations();

        try {
            InputStream stream = new FileInputStream(resource);
            ClassReader reader = new ClassReader(stream);
            TemplateAnnotationsCollector collector = new TemplateAnnotationsCollector(
                    new ClassWriter(true));
            reader.accept(collector, Attributes.getDefaultAttributes(), true);

            RuntimeVisibleAnnotations annotations = collector.getAnnotations();
            Iterator<?> it = annotations.annotations.iterator();

            while (it.hasNext()) {
                Annotation collectedAnnotation = (Annotation) it.next();
                if (isTemplateAnnotation(collectedAnnotation)) {
                    List<?> values = collectedAnnotation.elementValues;
                    Object [] annotatedClassValues = (Object []) values.get(0);
                    String annotatedClass = (String) annotatedClassValues[1];
                    Object [] elementValues = null;

                    if (values.size() > 1) {
                        elementValues = (Object []) values.get(1);
                    }

                    String classPath = getClassPath(path, annotatedClass);

                    InputStream annotationsHome = new FileInputStream(classPath);
                    ClassReader annotationsHomeReader = new ClassReader(
                            annotationsHome);

                    EquivalentAnnotationsCollector annotationsHomeCollector = new EquivalentAnnotationsCollector(
                            new ClassWriter(true));

                    if (getAnnotationName(collectedAnnotation.type).equals(FIELD_TEMPLATE)) {
                        annotationsHomeCollector.setFieldName((String) elementValues[1]);
                    } else if (getAnnotationName(collectedAnnotation.type).equals(METHOD_TEMPLATE)) {
                        annotationsHomeCollector.setMethodName((String) elementValues[1]);
                    } else if (getAnnotationName(collectedAnnotation.type).equals(CLASS_TEMPLATE)) {
                        annotationsHomeCollector.setClassAnnotations(true);
                    }

                    annotationsHomeReader.accept(annotationsHomeCollector,
                            Attributes.getDefaultAttributes(), true);

                    if (!annotationsHomeCollector.isElementVisited()
                            && !annotationsHomeCollector.isClassAnnotations()) {
                        String message;

                        if (getAnnotationName(collectedAnnotation.type).equals(FIELD_TEMPLATE)) {
                            message = "Field " + elementValues[1]
                                    + " could not be found in class "
                                    + classPath;
                        } else {
                            message = "Method " + elementValues[1]
                                    + " could not be found in class "
                                    + classPath;
                        }

                        throw new RuntimeException(message);
                    }

                    RuntimeVisibleAnnotations equivalentAnnotations = annotationsHomeCollector
                            .getAnnotations();
                    List<?> equivalentAnnotationsList = equivalentAnnotations.annotations;
                    Iterator<?> equivalentAnnotationsIterator = equivalentAnnotationsList
                            .iterator();

                    while (equivalentAnnotationsIterator.hasNext()) {
                        Annotation equivalentAnnotation = (Annotation) equivalentAnnotationsIterator
                                .next();
                        finalAnnotations.annotations.add(equivalentAnnotation);
                    }
                } else {
                    RuntimeVisibleAnnotations retrievedAnnotations = retrieveAnnotations(
                            path, collectedAnnotation);
                    List<?> retrievedAnnotationsList = retrievedAnnotations.annotations;
                    Iterator<?> retrievedAnnotationsListIterator = retrievedAnnotationsList
                            .iterator();
                    while (retrievedAnnotationsListIterator.hasNext()) {
                        Annotation nextRetrievedAnnotation = (Annotation) retrievedAnnotationsListIterator
                                .next();
                        List<?> finalAnnotationsList = finalAnnotations.annotations;
                        Iterator<?> finalAnnotationsIterator = finalAnnotationsList
                                .iterator();

                        while (finalAnnotationsIterator.hasNext()) {
                            Annotation existingAnnotation = (Annotation) finalAnnotationsIterator
                                    .next();
                            if (existingAnnotation.type
                                    .equals(nextRetrievedAnnotation.type)) {
                                throw new DuplicateAnnotationException(
                                        existingAnnotation);
                            }
                        }

                        finalAnnotations.annotations
                                .add(nextRetrievedAnnotation);
                    }
                }
            }
        } catch (FileNotFoundException exception) {
            if (isDomainAnnotation(path, annotation)) {
                String message = "It was not possible to retrive annotations of "
                        + "annotation "
                        + annotation.type
                        + " because file "
                        + resource + " does not exist.";
                throw new RuntimeException(message, exception);
            }
        } catch (IOException exception) {
            String annotationName = getAnnotationName(annotation.type);
            String message = "Annotations equivalent to domain annotation "
                    + annotationName
                    + " could not be retrieved due to the following reason: "
                    + exception.getMessage();
            throw new RuntimeException(message, exception);
        }

        return finalAnnotations;
    }

    /**
     * Verifies if an annotation is a template annotation, that is, if it is a
     * <code>ClassTemplate</code>, <code>FieldTemplate</code> or
     * <code>MethodTemplate</code> annotation.
     * 
     * @param collectedAnnotation
     *            The annotation to be verified.
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the given annotation is a template
     *         annotation;</li>
     *         <li><code>false</code> if the given annotation is not a template
     *         annotation.</li>
     *         </ul>
     */
    static boolean isTemplateAnnotation(Annotation collectedAnnotation) {
        boolean isTemplateAnnotation = false;
        String annotationName = getAnnotationName(collectedAnnotation.type);
        if (annotationName.equals(FIELD_TEMPLATE)
                || annotationName.equals(METHOD_TEMPLATE)
                || annotationName.equals(CLASS_TEMPLATE)) {
            isTemplateAnnotation = true;
        }
        return isTemplateAnnotation;
    }

    /**
     * Gets the physical path of a given annotation.
     * 
     * @param path
     *            The path to the annotation.
     * @param annotation
     *            The annotation itself.
     * @return The complete physical path to the annotation.
     */
    static String getResourcePath(String path, Annotation annotation) {
        path = path.replace('\\', '/');
        path = path.replace('.', '/');

        if (path.startsWith("/")) {
            path = path.substring(1);
        }

        if (!path.endsWith("/")) {
            path = path + "/";
        }

        String resource = path + getAnnotationName(annotation.type);
        resource = resource.replace('.', '/');
        resource += ".class";
        return resource;
    }

    /**
     * Gets the physical path of a given class.
     * 
     * @param path
     *            The path to the annotation.
     * @param className
     *            The fully-qualified name of the class.
     * @return The complete physical path to the class.
     */
    static String getClassPath(String path, String className) {
        path = path.replace('\\', '/');

        if (path.startsWith("/")) {
            path = path.substring(1);
        }

        if (!path.endsWith("/")) {
            path = path + "/";
        }

        String name = path + className;
        name = name.replace('.', '/');
        name += ".class";
        return name;
    }
}