package net.sf.daileon.notsupported;

import org.objectweb.asm.attrs.Annotation;

/**
 * The <code>Strategy</code> interface defines methods for evaluating the
 * annotations that correspond to a given domain annotation. The annotation
 * (represented by the <code>Annotation</code> object) is the domain annotation
 * to be evaluated, and this annotation annotates a given element (represented
 * by the <code>ElementRoot</code> object) and is located in a given path. The
 * <code>AbstractAnnotationsReplacer</code> object is responsible for keeping
 * the elements and their corresponding annotations, as well as their domain
 * annotations.
 * 
 * @author Beto
 * @version 1.0 01/01/2010
 */
public interface Strategy {

    /**
     * Sets the <code>AbstractAnnotationsReplacer</code> object. This object is
     * responsible for keeping the elements and their corresponding annotations,
     * as well as their domain annotations.
     * 
     * @param replacer
     *            The object that organizes elements and annotations.
     */
    public void setAnnotationsReplacer(AbstractAnnotationsReplacer replacer);

    /**
     * Sets the path of the domain annotation.
     * 
     * @param path
     *            The path of the domain annotation.
     */
    public void setPath(String path);

    /**
     * Sets the domain annotation.
     * 
     * @param annotation
     *            The domain annotation.
     */
    public void setAnnotation(Annotation annotation);

    /**
     * Sets the element that contains the domain annotation.
     * 
     * @param element
     *            The element that contains the domain annotation. This element
     *            can be a class, a field or a method.
     */
    public void setElement(ElementRoot element);

    /**
     * Triggers the process of evaluating a domain annotation. Essentially, this
     * method organizes the elements and their corresponding annotations, as
     * well as their domain annotations.
     * 
     * @throws IllegalStateException
     *             If any of the elements are not set in the implementation
     *             before starting the process of evaluating a domain
     *             annotation.
     * @throws RuntimeException
     *             If any other problem occur in the middle of the process of
     *             evaluating a domain annotation.
     * 
     */
    public void execute();

}