/*
 * 
 */
package net.sf.daileon.supported;

import java.lang.annotation.Annotation;
import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;
import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Set;

import net.sf.daileon.annotation.ClassTemplate;
import net.sf.daileon.annotation.DomainAnnotation;
import net.sf.daileon.annotation.FieldTemplate;
import net.sf.daileon.annotation.MethodTemplate;

/**
 * The <code>AnnotationsHelper</code> class provides static methods for dealing
 * with domain annotations. Essentially, it provides methods that inform if
 * domain annotations are annotated with known annotations of a particular
 * framework, as well as retrieving all annotations that annotate a particular
 * element.
 * 
 * @author Roberto Perillo
 * @author Eduardo Guerra
 * @version 1.0 02/12/2009
 */
public class DomainAnnotationsHelper<T> {

    /**
     * An empty array of <code>Annotation</code>, used by the
     * <code>getKnownAnnotation</code> method
     */
    private static final Annotation [] EMPTY_ANNOTATIONS_ARRAY = new Annotation[0];

    /**
     * Informs whether an {@link AnnotatedElement} is annotated with a
     * particular {@link Annotation}. This <code>Annotation</code> is searched
     * recursively in the given <code>AnnotatedElement</code>, which can be a
     * {@link Field}, a {@link Method} or even a {@link Class}. Therefore, it is
     * possible to verify if a class element, or even if a class, is annotated
     * with a known annotation.
     * 
     * @param element
     *            The <code>AnnotatedElement</code> (that is, the class element)
     *            to be verified.
     * @param annotationClass
     *            The <code>Annotation</code> to be verified.
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the given <code>AnnotatedElement</code>
     *         is annotated with a particular <code>Annotation</code>;</li>
     *         <li><code>false</code> if the given <code>AnnotatedElement</code>
     *         is not annotated with a particular <code>Annotation</code>.</li>
     *         </ul>
     */
    public static boolean isAnnotationPresent(final AnnotatedElement element,
            final Class<? extends Annotation> annotationClass) {

        if (element == null || annotationClass == null) {
            final String message = "Neither the AnnotatedElement argument or the "
                    + "Class<? extends Annotation> argument can be null.";
            throw new IllegalArgumentException(message);
        } else if (element.isAnnotationPresent(annotationClass)) {
            return true;
        } else {
            for (final Annotation annotation : element.getAnnotations()) {
                if (annotation.annotationType().equals(Documented.class)) {
                    return false;
                } else if (isAnnotationPresent(annotation.annotationType(),
                        annotationClass)) {
                    return true;
                }
            }
            return false;
        }
    }

    /**
     * <p>
     * Retrieves the known annotations of a given element. The known annotations
     * are the annotations that annotate an {@link AnnotatedElement} that is
     * also annotated with the {@link DomainAnnotation} annotation. For
     * instance:
     * </p>
     * <p>
     * 
     * <pre>
     * <code>@Retention(RetentionPolicy.<em>RUNTIME</em>)</code>
     * <code>@Target(ElementType.<em>METHOD</em>)</code>
     * <code>@FrameworkAnnotation1</code>
     * <code>@FrameworkAnnotation2</code>
     * <code>@DomainAnnotation</code>
     * public @interface ANewlyCreatedDomainAnnotation {}
     * 
     * <code>@ANewlyCreatedDomainAnnotation</code>
     * public void domainMethod() {
     *     // A business method
     * }
     * </pre>
     * 
     * </p>
     * <p>
     * In the example addressed above, <code>@FrameworkAnnotation1</code> and
     * <code>@FrameworkAnnotation2</code> are the annotations of the framework
     * or component that supports the creation of domain annotations; in other
     * words, they are the known annotations, which will be returned.
     * </p>
     * 
     * @param element
     *            An <code>AnnotatedElement</code> to be verified.
     * @return All known annotations (which are the annotations that annotate a
     *         domain annotation). Returns <code>null</code> if the
     *         <code>AnnotatedElement</code> is not annotated with
     *         <code>@DomainAnnotation</code>.
     */
    public static Annotation [] getKnownAnnotations(
            final AnnotatedElement element) {

        if (element == null) {
            final String message = "The AnnotatedElement argument cannot be null.";
            throw new IllegalArgumentException(message);
        }

        final Set<Annotation> knownAnnotations = new HashSet<Annotation>();
        final Annotation [] elementAnnotations = element.getAnnotations();

        for (final Annotation annotation : elementAnnotations) {
            final Class<? extends Annotation> annotationClass = annotation
                    .annotationType();
            if (isValidAnnotation(annotationClass)
                    && !isTemplateAnnotation(annotationClass)) {
                if (isAnnotationPresent(annotationClass, DomainAnnotation.class)) {
                    final Annotation [] annotations = getKnownAnnotations(annotation
                            .annotationType());
                    if (annotations != null) {
                        for (final Annotation knownAnnotation : annotations) {
                            knownAnnotations.add(knownAnnotation);
                        }
                    }
                } else {
                    knownAnnotations.add(annotation);
                }
            }
        }

        if (knownAnnotations.size() == 0) {
            return null;
        }

        return knownAnnotations.toArray(EMPTY_ANNOTATIONS_ARRAY);
    }

    /**
     * <p>
     * Retrieves the known annotations of an array of domain annotations. The
     * known annotations are the annotations that annotate each
     * {@link Annotation} that is annotated with the {@link DomainAnnotation}
     * annotation.
     * </p>
     * 
     * @param annotationsVerified
     *            An array of domain annotations.
     * @return All known annotations (which are the annotations that annotate a
     *         domain annotation). Returns <code>null</code> if no annotations
     *         are found the <code>AnnotatedElement</code> is not annotated with
     *         <code>@DomainAnnotation</code>.
     */
    public static Annotation [] getKnownAnnotations(
            final Annotation [] annotationsVerified) {

        if (annotationsVerified == null) {
            final String message = "The Annotation array argument cannot be null.";
            throw new IllegalArgumentException(message);
        }

        final Set<Annotation> knownAnnotations = new HashSet<Annotation>();

        for (final Annotation annotation : annotationsVerified) {
            final Annotation [] annotations = getKnownAnnotations(annotation
                    .annotationType());

            if (annotations != null) {
                for (final Annotation knownAnnotation : annotations) {
                    knownAnnotations.add(knownAnnotation);
                }
            }
        }

        if (knownAnnotations.size() == 0) {
            return null;
        }

        return knownAnnotations.toArray(EMPTY_ANNOTATIONS_ARRAY);
    }

    /**
     * Gets a particular annotation that indirectly annotates an
     * <code>AnnotatedElement</code>. For instance if
     * <code>@FrameworkAnnotation</code> annotates a domain annotation, and this
     * domain annotation annotates a particular element, then this method is
     * capable of retrieving <code>@FrameworkAnnotation</code> from the element
     * annotated with the domain annotation. If the searched annotation does not
     * indirectly annotates the element, then <code>null</code> is returned.
     * 
     * @param element
     *            The <code>AnnotatedElement</code>, annotated with a domain
     *            annotation.
     * @param annotationClass
     *            The <code>Class</code> of the annotation being searched.
     * @return The <code>Annotation</code> that indirectly annotates the given
     *         element, or <code>null</code> if the annotation is not found in
     *         the element.
     */
    @SuppressWarnings("unchecked")
    public static <T extends Annotation> T getAnnotation(
            final AnnotatedElement element, final Class<T> annotationClass) {
        final Annotation [] knownAnnotations = getKnownAnnotations(element);

        for (int i = 0; i < knownAnnotations.length; i++) {
            Annotation knownAnnotation = knownAnnotations[i];
            if (knownAnnotation.annotationType().equals(annotationClass)) {
                return (T) knownAnnotation;
            }
        }

        return null;
    }

    /**
     * Informs whether a particular <code>Annotation</code> is valid or not,
     * which can be {@link Documented}, {@link Retention}, {@link Target} or
     * {@link DomainAnnotation}.
     * 
     * @param annotationClass
     *            The <code>Class</code> that represents the annotation to be
     *            verified.
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the given <code>Annotation</code> is
     *         valid;</li>
     *         <li><code>false</code> if the given <code>Annotation</code> is
     *         not valid.</li>
     *         </ul>
     */
    private static boolean isValidAnnotation(
            Class<? extends Annotation> annotationClass) {
        return (!annotationClass.equals(Documented.class)
                && !annotationClass.equals(Retention.class)
                && !annotationClass.equals(Target.class) && !annotationClass
                .equals(DomainAnnotation.class));
    }

    /**
     * Informs whether a particular <code>Annotation</code> is a template
     * annotation, which can be {@link ClassTemplate}, {@link FieldTemplate} or
     * {@link MethodTemplate}.
     * 
     * @param annotationClass
     *            The <code>Class</code> that represents the annotation to be
     *            verified.
     * @return A <code>boolean</code> value that can be:
     *         <ul>
     *         <li><code>true</code> if the given <code>Annotation</code> is a
     *         template annotation;</li>
     *         <li><code>false</code> if the given <code>Annotation</code> is
     *         not a template annotation.</li>
     *         </ul>
     */
    private static boolean isTemplateAnnotation(
            Class<? extends Annotation> annotationClass) {
        return (annotationClass.equals(ClassTemplate.class)
                || annotationClass.equals(MethodTemplate.class) || annotationClass
                .equals(FieldTemplate.class));
    }
}