package net.sf.daileon.xml;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.xml.sax.SAXException;

/**
 * Represents the JAXB implementation of the <code>XmlHandler</code> interface.<br />
 * The <code>JaxbXmlHandler</code> is responsible for reading the XML
 * configuration file and retrieving an <code>Object</code> representation of
 * it, as well as serializing the content of a given <code>Object</code> (which
 * must be properly annotated with annotations of the JAXB 2.0 API) to a
 * particular {@link OutputStream}.
 * 
 * @author Beto
 * @version 1.0 01/01/2010
 * @see <a href="https://jaxb.dev.java.net/">The JAXB Project</a>
 */
public class JaxbXmlHandler implements XmlHandler {

    /** The path to the XSD schema of the XML configuration file */
    private static final String XSD_PATH = "net/sf/daileon/daileon-config.xsd";

    /**
     * Class constructor.
     */
    JaxbXmlHandler() {
        super();
    }

    /** {@inheritDoc} */
    public void beanToXml(final Object bean, final OutputStream destiny,
            final String jaxbContext) throws XmlHandlerException {
        try {
            final JAXBContext context = JAXBContext.newInstance(jaxbContext);
            final Marshaller marshaller = context.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
            marshaller.setProperty(Marshaller.JAXB_ENCODING, "iso-8859-1");
            marshaller.marshal(bean, destiny);
            destiny.flush();
            destiny.close();
        } catch (final JAXBException exception) {
            final String message = "The following error occurred while trying "
                    + "to transform the given bean to an xml representation: "
                    + exception.getMessage();
            throw new XmlHandlerException(message, exception);
        } catch (final IOException exception) {
            final String message = "The following error occurred while trying "
                    + "to transform the given bean to an xml representation: "
                    + exception.getMessage();
            throw new XmlHandlerException(message, exception);
        }
    }

    /** {@inheritDoc} */
    public Object xmlToBean(final InputStream source, final String jaxbContext)
            throws XmlHandlerException {
        try {
            URL url = getClass().getClassLoader().getResource(XSD_PATH);
            final SchemaFactory schemaFactory = SchemaFactory
                    .newInstance("http://www.w3.org/2001/XMLSchema");
            final Schema schema = schemaFactory.newSchema(url);

            final JAXBContext context = JAXBContext.newInstance(jaxbContext);
            final Unmarshaller unmarshaller = context.createUnmarshaller();
            unmarshaller.setSchema(schema);
            final Object bean = unmarshaller.unmarshal(source);
            source.close();
            return bean;
        } catch (final SAXException exception) {
            final String message = "The indicated XML schema is invalid. Please "
                    + "correct its path and try again.";
            throw new XmlHandlerException(message, exception);
        } catch (final JAXBException exception) {
            final String message = "The following error occurred while trying "
                    + "to transform the given xml structure to a bean "
                    + "representation: " + exception.getMessage();
            throw new XmlHandlerException(message, exception);
        } catch (final IOException exception) {
            final String message = "The following error occurred while trying "
                    + "to transform the given xml structure to a bean "
                    + "representation: " + exception.getMessage();
            throw new XmlHandlerException(message, exception);
        }
    }
}